import {IHighlightSettings} from './IHighlightSettings';

export class Highlight{

    protected readonly $element: Element;
    protected readonly highlightElements: HTMLDivElement[];
    protected readonly settings: Required<IHighlightSettings>;

    public get isHighlighted(): boolean{ return this.is_highlighted; }
    private is_highlighted: boolean = false;
    private interval: number|null = null;
    private timeout: number|null = null;

    /**
     * @param {Element} $element
     * @param {Required<IHighlightSettings>} settings
     */
    public constructor($element: Element, settings: Required<IHighlightSettings>){
        this.$element = $element;
        this.highlightElements = [
            document.createElement('div'), // left
            document.createElement('div'), // top
            document.createElement('div'), // right
            document.createElement('div'), // bottom
        ];
        this.settings = settings;

        this.add = this.add.bind(this);
        this.remove = this.remove.bind(this);
        this.draw = this.draw.bind(this);
    }

    public add(): void{
        if(this.is_highlighted)
            return;

        this.is_highlighted = true;
        for(const $el of this.highlightElements){
            $el.style.display = 'block';
            $el.style.opacity = '1';
            $el.style.visibility = 'visible';
            this.$element.appendChild($el);
        }

        this.draw();
        this.interval = setInterval(this.draw, 250);

        if(this.settings.hideAfter > 0 && this.settings.hideAfter !== Infinity)
            this.timeout = setTimeout(this.remove, this.settings.hideAfter);
    }

    public remove(): void{
        if(!this.is_highlighted)
            return;

        this.is_highlighted = false;

        if(this.interval !== null){
            clearInterval(this.interval);
            this.interval = null;
        }

        if(this.timeout !== null){
            clearTimeout(this.timeout);
            this.timeout = null;
        }

        for(const $el of this.highlightElements){
            $el.style.display = 'none';
            $el.style.opacity = '0';
            $el.style.visibility = 'hidden';
            $el.remove();
        }
    }

    /**
     * @protected
     */
    protected draw(): void{
        const targetRect = this.$element.getBoundingClientRect(),
            pageX = window.pageXOffset,
            pageY = window.pageYOffset;

        for(const $el of this.highlightElements){
            $el.style.position = 'fixed';
            $el.style.backgroundColor = this.settings.strokeColor;
        }

        // Left
        this.highlightElements[0].style.width = this.settings.strokeSize + 'px';
        this.highlightElements[0].style.height = targetRect.height + 'px';
        this.highlightElements[0].style.left = (targetRect.x + pageX) + 'px';
        this.highlightElements[0].style.top = (targetRect.y + pageY) + 'px';

        // Right
        this.highlightElements[2].style.width = this.settings.strokeSize + 'px';
        this.highlightElements[2].style.height = targetRect.height + 'px';
        this.highlightElements[2].style.left = (targetRect.x + targetRect.width + pageX) + 'px';
        this.highlightElements[2].style.top = (targetRect.y + pageY) + 'px';

        // Top
        this.highlightElements[1].style.height = this.settings.strokeSize + 'px';
        this.highlightElements[1].style.width = targetRect.width + 'px';
        this.highlightElements[1].style.left = (targetRect.x + pageX) + 'px';
        this.highlightElements[1].style.top = (targetRect.y + pageY) + 'px';

        // Bottom
        this.highlightElements[3].style.height = this.settings.strokeSize + 'px';
        this.highlightElements[3].style.width = targetRect.width + 'px';
        this.highlightElements[3].style.left = (targetRect.x + pageX) + 'px';
        this.highlightElements[3].style.top = (targetRect.y + targetRect.height + pageY) + 'px';
    }

}
